
<!-- README.md is generated from README.Rmd. Please edit that file -->

# odds

<!-- badges: start -->

<!-- badges: end -->

The goal of `{odds}` is to provide an on-disk data-storage of native R
object, for cross-session data access.

## Installation

You can install the dev version of `{odds}` from GitHub with:

``` r
remotes::install_github("colinfay/odds")
```

## Basic use

The main goal of `{odds}` is to create a data storage architecture, on
disk, so that you can access the values from one session to another.

### How it works

By default, the storage is done at `~/.odds`, but it can be changed when
creating the storage object.

**Note that the path is passed through `fs::path_norm()`, which doesn’t
treat `~` the same way as base R on Windows.**

``` r
library(odds)
st <- Storage$new()
```

There are two main methods: `set()` and `get()`. The first saves a value
under a name on the disk, the second retrieve this value from the
storage.

``` r
st$set(head(iris), "a")
st$get("a")
#>   Sepal.Length Sepal.Width Petal.Length Petal.Width Species
#> 1          5.1         3.5          1.4         0.2  setosa
#> 2          4.9         3.0          1.4         0.2  setosa
#> 3          4.7         3.2          1.3         0.2  setosa
#> 4          4.6         3.1          1.5         0.2  setosa
#> 5          5.0         3.6          1.4         0.2  setosa
#> 6          5.4         3.9          1.7         0.4  setosa
```

Storages can be namespaced, and the default is “global”,

``` r
nsp <- paste(sample(letters, 3), collapse = "")

st$set(mtcars, "a", namespace = nsp)
st$get("a", namespace = nsp)
#>                      mpg cyl  disp  hp drat    wt  qsec vs am gear carb
#> Mazda RX4           21.0   6 160.0 110 3.90 2.620 16.46  0  1    4    4
#> Mazda RX4 Wag       21.0   6 160.0 110 3.90 2.875 17.02  0  1    4    4
#> Datsun 710          22.8   4 108.0  93 3.85 2.320 18.61  1  1    4    1
#> Hornet 4 Drive      21.4   6 258.0 110 3.08 3.215 19.44  1  0    3    1
#> Hornet Sportabout   18.7   8 360.0 175 3.15 3.440 17.02  0  0    3    2
#> Valiant             18.1   6 225.0 105 2.76 3.460 20.22  1  0    3    1
#> Duster 360          14.3   8 360.0 245 3.21 3.570 15.84  0  0    3    4
#> Merc 240D           24.4   4 146.7  62 3.69 3.190 20.00  1  0    4    2
#> Merc 230            22.8   4 140.8  95 3.92 3.150 22.90  1  0    4    2
#> Merc 280            19.2   6 167.6 123 3.92 3.440 18.30  1  0    4    4
#> Merc 280C           17.8   6 167.6 123 3.92 3.440 18.90  1  0    4    4
#> Merc 450SE          16.4   8 275.8 180 3.07 4.070 17.40  0  0    3    3
#> Merc 450SL          17.3   8 275.8 180 3.07 3.730 17.60  0  0    3    3
#> Merc 450SLC         15.2   8 275.8 180 3.07 3.780 18.00  0  0    3    3
#> Cadillac Fleetwood  10.4   8 472.0 205 2.93 5.250 17.98  0  0    3    4
#> Lincoln Continental 10.4   8 460.0 215 3.00 5.424 17.82  0  0    3    4
#> Chrysler Imperial   14.7   8 440.0 230 3.23 5.345 17.42  0  0    3    4
#> Fiat 128            32.4   4  78.7  66 4.08 2.200 19.47  1  1    4    1
#> Honda Civic         30.4   4  75.7  52 4.93 1.615 18.52  1  1    4    2
#> Toyota Corolla      33.9   4  71.1  65 4.22 1.835 19.90  1  1    4    1
#> Toyota Corona       21.5   4 120.1  97 3.70 2.465 20.01  1  0    3    1
#> Dodge Challenger    15.5   8 318.0 150 2.76 3.520 16.87  0  0    3    2
#> AMC Javelin         15.2   8 304.0 150 3.15 3.435 17.30  0  0    3    2
#> Camaro Z28          13.3   8 350.0 245 3.73 3.840 15.41  0  0    3    4
#> Pontiac Firebird    19.2   8 400.0 175 3.08 3.845 17.05  0  0    3    2
#> Fiat X1-9           27.3   4  79.0  66 4.08 1.935 18.90  1  1    4    1
#> Porsche 914-2       26.0   4 120.3  91 4.43 2.140 16.70  0  1    5    2
#> Lotus Europa        30.4   4  95.1 113 3.77 1.513 16.90  1  1    5    2
#> Ford Pantera L      15.8   8 351.0 264 4.22 3.170 14.50  0  1    5    4
#> Ferrari Dino        19.7   6 145.0 175 3.62 2.770 15.50  0  1    5    6
#> Maserati Bora       15.0   8 301.0 335 3.54 3.570 14.60  0  1    5    8
#> Volvo 142E          21.4   4 121.0 109 4.11 2.780 18.60  1  1    4    2
```

### Cross session access

Let’s create an object in another R session:

``` r
library(callr)
rx <- r_bg(
  function(){
    library(odds)
    st <- Storage$new()
    st$set(head(airquality), "ping", namespace = "blop")
  }
)
```

It’s now accessible in the first session:

``` r
st$get("ping", namespace = "blop")
#>   Ozone Solar.R Wind Temp Month Day
#> 1    41     190  7.4   67     5   1
#> 2    36     118  8.0   72     5   2
#> 3    12     149 12.6   74     5   3
#> 4    18     313 11.5   62     5   4
#> 5    NA      NA 14.3   56     5   5
#> 6    28      NA 14.9   66     5   6
```

Values can be deleted:

``` r
st$rm("ping", namespace = "blop")
```

Namespaces can be deleted:

``` r
st$remove_namespace(nsp)
st$remove_namespace("blop")
```

## Overhead

Of course, reading from disk adds some overhead, but for small to medium
size objects, the cost of `get`ting from disk instead of reading for RAM
is pretty small.

``` r
library(dplyr, warn.conflicts = FALSE)
library(ggplot2)
st$set(diamonds, "dm", "bench")
bench::mark(
  ram = {
   diamonds %>% filter(cut == "Ideal")
  }, 
  disk = {
    st$get("dm", "bench") %>% 
      filter(cut == "Ideal")
  }
)
#> # A tibble: 2 x 6
#>   expression      min   median `itr/sec` mem_alloc `gc/sec`
#>   <bch:expr> <bch:tm> <bch:tm>     <dbl> <bch:byt>    <dbl>
#> 1 ram           1.5ms   3.01ms     287.     3.07MB     21.7
#> 2 disk         12.7ms  14.55ms      66.7    6.05MB     12.8
```

`set()` and `get()` are powered by `{qs}` `qread()` and `qwrite()` and
take the same arguments, so you can use parameters to these functions to
speed up the read and write timing.

Read the `{qs}` benchmark
[online](https://github.com/traversc/qs#summary-table).

## Acknowledgment

This package heavily relies on the `{qs}` package. Thanks to the package
authors for their work.

## Coc

Please note that the ‘odds’ project is released with a [Contributor Code
of Conduct](CODE_OF_CONDUCT.md). By contributing to this project, you
agree to abide by its terms.
